<?php
/*
** Zabbix
** Copyright (C) 2001-2019 Zabbix SIA
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/


header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Headers: Content-Type');
header('Access-Control-Allow-Methods: POST');
header('Access-Control-Max-Age: 1000');

if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
	return;
}

require_once dirname(__FILE__).'/include/func.inc.php';
//ELISA start
require_once dirname(__FILE__).'/include/defines.inc.php';
require_once dirname(__FILE__).'/include/items.inc.php';
require_once dirname(__FILE__).'/include/hosts.inc.php';
//ELISA end
require_once dirname(__FILE__).'/include/classes/core/CHttpRequest.php';

$allowed_content = [
	'application/json-rpc' => 'json-rpc',
	'application/json' => 'json-rpc',
	'application/jsonrequest' => 'json-rpc',
];
$http_request = new CHttpRequest();
$content_type = $http_request->header('Content-Type');
$content_type = explode(';', $content_type);
$content_type = $content_type[0];

if (!isset($allowed_content[$content_type])) {
	header('HTTP/1.0 412 Precondition Failed');
	return;
}

require_once dirname(__FILE__).'/include/classes/core/Z.php';

header('Content-Type: application/json');
$data = $http_request->body();

try {
	Z::getInstance()->run(ZBase::EXEC_MODE_API);

	$apiClient = API::getWrapper()->getClient();

	// unset wrappers so that calls between methods would be made directly to the services
	API::setWrapper();

	$jsonRpc = new CJsonRpc($apiClient, $data);
	// ELISA start
	$datajson = json_decode($data);
	$apiuser=0;
	$db_users = DB::select('sessions', [
		'output' => ['userid'],
		'filter' => ['sessionid' => $datajson->{'auth'}]
	]);
	if(count($db_users)==1){
		$apiuser=$db_users[0]['userid'];
	}
	$item_old = [];
	$host = [];
	if($datajson->{'method'}=="item.update"){
		$item_old[$datajson->{'params'}->{'itemid'}] = get_item_by_itemid($datajson->{'params'}->{'itemid'});
		$host[$datajson->{'params'}->{'itemid'}] = get_host_by_hostid($item_old[$datajson->{'params'}->{'itemid'}]['hostid']);
	}else if($datajson->{'method'}=="item.delete"){
		foreach($datajson->{'params'} as $itemid){
			$item_old[$itemid] = get_item_by_itemid($itemid);
			$host[$itemid] = get_host_by_hostid($item_old[$itemid]['hostid']);
		}
	}	
	// ELISA end
	
	$result = $jsonRpc->execute();
	echo $result;
	
	//ELISA start
	try {
		if($datajson->{'method'}=="item.create"){
			$resultjson = json_decode($result);
			foreach($resultjson->{'result'}->{'itemids'} as $itemid){
				$item_new = get_item_by_itemid($itemid);
				$log = add_audit_elisa(AUDIT_ACTION_ADD, AUDIT_RESOURCE_ITEM, $itemid, "key=".$item_old[$itemid]["key_"].", host: ".$host[$itemid]["host"], "items", null, $item_new, $apiuser);
			}
		}else if($datajson->{'method'}=="item.update"){
			$resultjson = json_decode($result);
			foreach($resultjson->{'result'}->{'itemids'} as $itemid){
				$item_new = get_item_by_itemid($itemid);
				$log = add_audit_elisa(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_ITEM, $itemid, "key=".$item_old[$itemid]["key_"].", host: ".$host[$itemid]["host"], "items", $item_old[$itemid], $item_new, $apiuser);
			}
		}else if($datajson->{'method'}=="item.delete"){
			$resultjson = json_decode($result);
			foreach($resultjson->{'result'}->{'itemids'} as $itemid){
				$log = add_audit_elisa(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_ITEM, $itemid, "key=".$item_old[$itemid]["key_"].", host: ".$host[$itemid]["host"], "items", $item_old[$itemid], null, $apiuser);
			}
			
		}
	}catch (Exception $e_elisa) {}
	//ELISA end
	
	
}
catch (Exception $e) {
	// decode input json request to get request's id
	$jsonData = CJs::decodeJson($data);

	$response = [
		'jsonrpc' => '2.0',
		'error' => [
			'code' => 1,
			'message' => $e->getMessage(),
			'data' => ''
		],
		'id' => (isset($jsonData['id']) ? $jsonData['id'] : null)
	];

	echo CJs::encodeJson($response);
}
