<?php
/*
** Zabbix
** Copyright (C) 2001-2015 Zabbix SIA
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

define('ELISA_CONF_GUI','/usr/share/elisa/conf/elisa.gui.conf');
define('ELISA_CONF','/usr/share/elisa/conf/elisa.conf');


require_once dirname(__FILE__).'/include/config.inc.php';
require_once dirname(__FILE__).'/include/js.inc.php';
require_once dirname(__FILE__).'/include/users.inc.php';
require_once dirname(__FILE__).'/include/forms.inc.php';

$config = select_config();
$userdata = getUserFormData(CWebUser::$data['userid'], $config, true);
if($userdata['lang'] == "cs_CZ"){
	define('ELISA_CONF_HELP','/usr/share/elisa/conf/elisa.gui.help.cs');
}else{
	define('ELISA_CONF_HELP','/usr/share/elisa/conf/elisa.gui.help');
}



$page['title'] = _('Archive');
$page['file'] = 'elisa_archive.php';
$page['hist_arg'] = array();

require_once dirname(__FILE__).'/include/page_header.php';

$themes = array_keys(Z::getThemes());
$themes[] = THEME_DEFAULT;

// VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
$fields = array(
	'nfs' => 					array(T_ZBX_STR, O_OPT, null, null, null),
	'DATA_RETENTION_COMPRESS' => 					array(T_ZBX_STR, O_OPT, null, null, null),
	'DATA_RETENTION_ARCHIVEDELETE' => 					array(T_ZBX_STR, O_OPT, null, null, null),
	// actions
	'update'=>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null),
	'umount'=>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null),
	'mount'=>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null),
	'indicesid' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
	'action' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
									IN('"indices.massrestore"'),
									null
								],
	'form_refresh' =>			array(T_ZBX_INT, O_OPT,	null,	null,	null)
);
check_fields($fields);

/*
 * Actions
 */
$result = true;
$result1 = true;
if (hasRequest('umount')) {
	$umount = exec('sudo /usr/share/elisa/bin/elisa-umountArchive.sh', $output, $rc1);
	add_audit_details(AUDIT_ACTION_DELETE, 99, 0, '', 'indices-archive-nfs');
}else if (hasRequest('mount')) {
	$mount = exec('sudo /usr/share/elisa/bin/elisa-mountArchive.sh "'.getRequest('nfs').'"', $output, $rc1);
	$ok = true;	if($rc1!=0){$ok = false;}
	show_messages($ok, _('NFS successfully mounted'), _('Error occured during NFS mount').': '.$output[0]);	
	add_audit_details(AUDIT_ACTION_ADD, 99, 0, getRequest('nfs'), 'indices-archive-nfs');
}

if (hasRequest('update')) {
	$conf = array();
	$file = fopen(ELISA_CONF_GUI,"r");
	while(! feof($file)){
		$line = fgets($file);
		if(!is_int(strpos($line, '#')) && trim($line) != ""){
			$lineArr = explode("=",$line);
			$key = trim($lineArr[0]);
			$value = trim($lineArr[1]);
			$conf[$key] = $value;
		}
	}
	fclose($file);	
	$conf['DATA_RETENTION_COMPRESS']=getRequest('DATA_RETENTION_COMPRESS');
	$conf['DATA_RETENTION_ARCHIVEDELETE']=getRequest('DATA_RETENTION_ARCHIVEDELETE');
	
	$fileContent = '';
	foreach ($conf as $key => $value) {
		$fileContent .= $key."=".$value."\n";
		add_audit_details(AUDIT_ACTION_UPDATE, 99, 0, $key."=".$value, 'indices-archive-settings');
	}	
	
	$result = file_put_contents(ELISA_CONF_GUI,$fileContent);
}

if (hasRequest('action') && getRequest('action') == 'indices.massrestore' && hasRequest('indicesid')) {
	$indicesIds = getRequest('indicesid');
	zbx_value2array($indicesIds);
	$rc = true;
	$cmds1 = [];
	$cmds2 = [];
	foreach ($indicesIds as $name => $value) {
		$cmds1[] = 'sudo /usr/share/elisa/bin/elisa-indiceRestoreTouchWaiting.sh '.$name;
		$cmds2[] = 'sudo /usr/share/elisa/bin/elisa-indiceRestore.sh '.$name;
		add_audit_details(AUDIT_ACTION_ADD, 99, 0, $key."=".$value, 'indices-archive-restore');
	}
	$cmd1 = implode(";",$cmds1);
	$cmd1 = '('.$cmd1.') >/dev/null 2>/dev/null';
	$cmd2 = implode(";",$cmds2);
	$cmd2 = '('.$cmd2.') >/dev/null 2>/dev/null &';
	exec($cmd1, $output, $rci);
	exec($cmd2, $output, $rci);
	if($rci > 0){$rc = false;}
	show_messages($rc, _('Indices started to be restores. Wait few minutes...'), _('Cannot restore indices'));	
	uncheckTableRows('indicesid');
	sleep(2);
}

/*
 * Display
 */
 
$data = array();

$DATA_PATH_COMPRESS = shell_exec('grep -i DATA_PATH_COMPRESS= '.ELISA_CONF.' | cut -d= -f2 | cut -d# -f1');
define('DATA_PATH_COMPRESS',trim($DATA_PATH_COMPRESS));
$NFS = explode(' ',shell_exec('df -h '.DATA_PATH_COMPRESS.' | grep '.DATA_PATH_COMPRESS.' | tr -s " " " " '));
if(count($NFS)==6){
	$data['config']['nfs'] = $NFS[0];
	$data['config']['nfsavail'] = $NFS[3];
}
else{
	$data['config']['nfs'] = '';
	$data['config']['nfsavail'] = '';
}
$data['help']['nfs'] = [
	'category' => _('Configuration'),
	'label' => _('NFS server:path'),
	'hint' => _('NFS share for storing data archive.'),
	'htmlType' => 'textbox'
];
$data['help']['nfsavail'] = [
	'category' => _('Configuration'),
	'label' => _('NFS available size'),
	'hint' => null,
	'htmlType' => 'label'
];


$file = fopen(ELISA_CONF_GUI,"r");
while(! feof($file)){
	$line = fgets($file);
	if(!is_int(strpos($line, '#')) && trim($line) != ""){
		$lineArr = explode("=",$line);
		$key = trim($lineArr[0]);
		$value = trim($lineArr[1]);
		if($key=="DATA_RETENTION_COMPRESS"){$data['config'][$key] = $value;}
		else if($key=="DATA_RETENTION_ARCHIVEDELETE"){$data['config'][$key] = $value;}
	}
}
fclose($file);

$file = fopen(ELISA_CONF_HELP,"r");
while(! feof($file)){
	$line = fgets($file);
	if(strpos($line, '#') == FALSE && trim($line) != ""){
		$lineArr = explode(";",$line);
		$key = trim($lineArr[0]);
		$category = trim($lineArr[1]);
		$label = trim($lineArr[2]);
		if(count($lineArr)>3){$hint = trim($lineArr[3]);}else{$hint='';}
		if(count($lineArr)>4){$htmlType = trim($lineArr[4]);}else{$htmlType='text';}
		$data['help'][$key] = array(
			'category' => _('Configuration'),
			'label' => $label,
			'hint' => $hint,
			'htmlType' => $htmlType
		);
	}
}
fclose($file);

$data['indice'] = [];
$dataDirs = scandir(DATA_PATH_COMPRESS);
foreach ($dataDirs as $file) {
	if( preg_match( '/.*-\d\d\d\d\.\d\d.\d\d\.tgz.arch$/', $file) && is_file(DATA_PATH_COMPRESS.'/'.$file)){
		$fileinfo = pathinfo(DATA_PATH_COMPRESS.'/'.$file);
		$fileSize = filesize(DATA_PATH_COMPRESS.'/'.$file);
		$indiceName = str_replace('.tgz.arch','',$file);
		if($fileSize){ $fileSize = $fileSize; }
		if(! array_key_exists($file,$data['indice']) ){ 
			$data['indice'][$indiceName]['name'] = $indiceName;
			$data['indice'][$indiceName]['creationDate'] = filemtime(DATA_PATH_COMPRESS.'/'.$file)*1000;
			$data['indice'][$indiceName]['size_in_bytes'] = $fileSize;
		}
	}
}	
krsort($data['indice']);




// render view

$configView = new CView('elisa.archive', $data);
$configView->render();
$configView->show();




require_once dirname(__FILE__).'/include/page_footer.php';
