<?php
/*
** Zabbix
** Copyright (C) 2001-2015 Zabbix SIA
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

define('ELISA_CONF_GUI','/usr/share/elisa/conf/elisa.gui.conf');
define('ELISA_CONF','/usr/share/elisa/conf/elisa.conf');


require_once dirname(__FILE__).'/include/config.inc.php';
require_once dirname(__FILE__).'/include/js.inc.php';
require_once dirname(__FILE__).'/include/users.inc.php';
require_once dirname(__FILE__).'/include/forms.inc.php';

$config = select_config();
$userdata = getUserFormData(CWebUser::$data['userid'], $config, true);
if($userdata['lang'] == "cs_CZ"){
	define('ELISA_CONF_HELP','/usr/share/elisa/conf/elisa.gui.help.cs');
}else{
	define('ELISA_CONF_HELP','/usr/share/elisa/conf/elisa.gui.help');
}



$page['title'] = _('Backup');
$page['file'] = 'elisa_backup.php';
$page['hist_arg'] = array();

require_once dirname(__FILE__).'/include/page_header.php';

$themes = array_keys(Z::getThemes());
$themes[] = THEME_DEFAULT;

// VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
$fields = array(
	'nfs' => 					array(T_ZBX_STR, O_OPT, null, null, null),
	'BACKUP_DELETE' => 					array(T_ZBX_STR, O_OPT, null, null, null),
	'BACKUP_ZABBIX_HISTORY_DATA' => 	array(T_ZBX_STR, O_OPT, null, null, null),
	// actions
	'update'=>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null),
	'umount'=>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null),
	'mount'=>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null),
	'backupid' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
	'restore'=>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null),
	'backupnow'=>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null),
	'form_refresh' =>			array(T_ZBX_INT, O_OPT,	null,	null,	null)
);
check_fields($fields);

/*
 * Actions
 */
$result = true;
$result1 = true;
if (hasRequest('umount')) {
	$umount = exec('sudo /usr/share/elisa/bin/elisa-umountBackup.sh', $output, $rc1);
	add_audit_details(AUDIT_ACTION_DELETE, 99, 0, '', 'backup-nfs');
}else if (hasRequest('mount')) {
	$mount = exec('sudo /usr/share/elisa/bin/elisa-mountBackup.sh "'.getRequest('nfs').'"', $output, $rc1);
	$ok = true;	if($rc1!=0){$ok = false;}
	show_messages($ok, _('NFS successfully mounted'), _('Error occured during NFS mount').': '.$output[0]);	
	add_audit_details(AUDIT_ACTION_ADD, 99, 0, getRequest('nfs'), 'backup-nfs');
}
if (hasRequest('backupnow')) {
	exec('sudo /usr/share/elisa/bin/elisa-systemBackup.sh >/dev/null 2>/dev/null &', $output, $rc1);
	show_messages(true, _('System backup initiated. It can take few minutes till be created'), _('Error occured during initiating backup'));	
	add_audit_details(AUDIT_ACTION_ADD, 99, 0, getRequest('nfs'), 'backup-create');
}else if (hasRequest('update')) {
	$conf = array();
	$file = fopen(ELISA_CONF_GUI,"r");
	while(! feof($file)){
		$line = fgets($file);
		if(!is_int(strpos($line, '#')) && trim($line) != ""){
			$lineArr = explode("=",$line);
			$key = trim($lineArr[0]);
			$value = trim($lineArr[1]);
			$conf[$key] = $value;
		}
	}
	fclose($file);	
	$conf['BACKUP_DELETE']=getRequest('BACKUP_DELETE');
	$conf['BACKUP_ZABBIX_HISTORY_DATA']=getRequest('BACKUP_ZABBIX_HISTORY_DATA');
	
	
	$fileContent = '';
	foreach ($conf as $key => $value) {
		$fileContent .= $key."=".$value."\n";
		add_audit_details(AUDIT_ACTION_UPDATE, 99, 0, $key."=".$value, 'backup-settings');
	}	
	
	$result = file_put_contents(ELISA_CONF_GUI,$fileContent);
	
}

if (hasRequest('restore') && hasRequest('backupid')) {
	$backupid = getRequest('backupid');
	$rc = true;
	$cmd1 = 'sudo /usr/share/elisa/bin/elisa-systemRestore.sh '.$backupid.' >/dev/null 2>/dev/null &';
	exec($cmd1, $output, $rci);
	if($rci > 0){$rc = false;}
	show_messages($rc, _('System Restore initiated. Wait few minutes till system restart.'), _('Cannot restore system.'));	
	add_audit_details(AUDIT_ACTION_ADD, 99, 0, $backupid, 'backup-restore');
	sleep(2);
}

/*
 * Display
 */
 
$data = array();

$BACKUP_PATH = shell_exec('grep -i BACKUP_PATH= '.ELISA_CONF.' | cut -d= -f2 | cut -d# -f1');
define('BACKUP_PATH',trim($BACKUP_PATH));

$NFS = explode(' ',shell_exec('df -h '.BACKUP_PATH.' | grep '.BACKUP_PATH.' | tr -s " " " " '));
if(count($NFS)==6){
	$data['config']['nfs'] = $NFS[0];
	$data['config']['nfsavail'] = $NFS[3];
	$data['config']['localpath'] = false;
}
else{
	$data['config']['nfs'] = '';
	$data['config']['nfsavail'] = '';
	$data['config']['localpath'] = true;
}
$data['help']['nfs'] = [
	'category' => _('Configuration'),
	'label' => _('NFS server:path'),
	'hint' => _('NFS share for storing system backups.'),
	'htmlType' => 'textbox'
];
$data['help']['nfsavail'] = [
	'category' => _('Configuration'),
	'label' => _('NFS available size'),
	'hint' => null,
	'htmlType' => 'label'
];
$data['help']['localpath'] = [
	'category' => _('Configuration'),
	'label' => _('Local path'),
	'hint' => _('System backups are stored locally.'),
	'htmlType' => 'textbox'
];


$file = fopen(ELISA_CONF_GUI,"r");
while(! feof($file)){
	$line = fgets($file);
	if(!is_int(strpos($line, '#')) && trim($line) != ""){
		$lineArr = explode("=",$line);
		$key = trim($lineArr[0]);
		$value = trim($lineArr[1]);
		if($key=="BACKUP_DELETE"){$data['config'][$key] = $value;}
		if($key=="BACKUP_ZABBIX_HISTORY_DATA"){$data['config'][$key] = $value;}
	}
}
fclose($file);

$file = fopen(ELISA_CONF_HELP,"r");
while(! feof($file)){
	$line = fgets($file);
	if(strpos($line, '#') == FALSE && trim($line) != ""){
		$lineArr = explode(";",$line);
		$key = trim($lineArr[0]);
		$category = trim($lineArr[1]);
		$label = trim($lineArr[2]);
		if(count($lineArr)>3){$hint = trim($lineArr[3]);}else{$hint='';}
		if(count($lineArr)>4){$htmlType = trim($lineArr[4]);}else{$htmlType='text';}
		$data['help'][$key] = array(
			'category' => _('Configuration'),
			'label' => $label,
			'hint' => $hint,
			'htmlType' => $htmlType
		);
	}
}
fclose($file);

$data['files'] = [];
$backupFiles = scandir(BACKUP_PATH.'/system');
foreach ($backupFiles as $file) {
	if( preg_match( '/.*.backup.tar$/', $file) && is_file(BACKUP_PATH.'/system/'.$file)){
		$fileinfo = pathinfo(BACKUP_PATH.'/system/'.$file);
		$fileSize = filesize(BACKUP_PATH.'/system/'.$file);
		$fileName = str_replace('.tar','',$file);
		if($fileSize){ $fileSize = $fileSize; }
		if(! array_key_exists($file,$data['files']) ){ 
			$data['files'][$fileName]['name'] = $fileName;
			$data['files'][$fileName]['creationDate'] = filemtime(BACKUP_PATH.'/system/'.$file)*1000;
			$data['files'][$fileName]['size_in_bytes'] = $fileSize;
		}
	}
}	
krsort($data['files']);



// render view

$configView = new CView('elisa.backup', $data);
$configView->render();
$configView->show();




require_once dirname(__FILE__).'/include/page_footer.php';
