<?php
/*
** Zabbix
** Copyright (C) 2001-2015 Zabbix SIA
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/


require_once dirname(__FILE__).'/include/config.inc.php';
require_once dirname(__FILE__).'/include/js.inc.php';


$conf = [];
$conf['templateServer'] = 'Template-DS-Xlog_ELISA_Server';
$conf['itemAlarm'] = 'xlog.config[AGENT,Processor,AlarmProcessor,950,SyslogForwarder]';


$page['title'] = _('Event Forwarding Configuration');
$page['file'] = 'elisa_forward.php';
$page['hist_arg'] = array();

require_once dirname(__FILE__).'/include/page_header.php';

$themes = array_keys(Z::getThemes());
$themes[] = THEME_DEFAULT;

// VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
$fields = array(
	'forwardsysloghost' => [T_ZBX_STR, O_OPT, null, null, null],
	'forwardsyslogport' => [T_ZBX_STR, O_OPT, null, null, null],
	'forwardsyslogenable' => [T_ZBX_STR, O_OPT, null, null, null],
	'forwardsyslogprotocol' => [T_ZBX_STR, O_OPT, null, null, null],
	'update' => [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null, null],
	'cancel' => [T_ZBX_STR, O_OPT, P_SYS, null, null]
);
check_fields($fields);




	
/*
 * Actions
 */

if(hasRequest('update')){
	if( getRequest('forwardsyslogenable',0) == 1){
		if(getRequest('forwardsysloghost','')!='' && getRequest('forwardsyslogport','')!=''){
			//print "enable<br>";
			$templates = API::Template()->get([
				'output' => API_OUTPUT_EXTEND,
				'search' => [
					'name' => $conf['templateServer']
				],
				'selectMacros' => API_OUTPUT_EXTEND,
				'limit' => 1
			]);
			if(count($templates)==1){
				foreach($templates[0]['macros'] as $macro){
					if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_HOST}'){API::UserMacro()->update(['hostmacroid'=>$macro['hostmacroid'],'value'=>getRequest('forwardsysloghost','')]); }
					if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_PORT}'){API::UserMacro()->update(['hostmacroid'=>$macro['hostmacroid'],'value'=>getRequest('forwardsyslogport','')]); }
					if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_MODULE}'){API::UserMacro()->update(['hostmacroid'=>$macro['hostmacroid'],'value'=> getRequest('forwardsyslogprotocol',0)==1 ? 'om_tcp' : 'om_udp' ]); }
					if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_FOO}'){API::UserMacro()->update(['hostmacroid'=>$macro['hostmacroid'],'value'=> $macro['value']=='' ? 'nejaka nahodna hodnota, aby doslo k reload konfigurace' : '' ]); }
				}		
				$itemSyslogForward = API::Item()->get([
					'output' => ['itemid','key_','status'], //API_OUTPUT_EXTEND,
					'search' => [
						'key_' => 'SyslogForwarder'
					],
					'hostids' => $templates[0]['templateid'],
					'limit' => 10
				]);	
				foreach($itemSyslogForward as $item){			
					API::Item()->update(['itemid'=>$item['itemid'],'status'=>0]);
				}
				show_messages(true, _('Syslog forwarding was successfully changed'), '');
			}
		}else{
			show_messages(false, '', _('Set the host and port please'));
		}
	}else{
		$templates = API::Template()->get([
			'output' => API_OUTPUT_EXTEND,
			'search' => [
				'name' => $conf['templateServer']
			],
			'selectMacros' => API_OUTPUT_EXTEND,
			'limit' => 1
		]);
		if(count($templates)==1){
			foreach($templates[0]['macros'] as $macro){
				if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_HOST}'){API::UserMacro()->update(['hostmacroid'=>$macro['hostmacroid'],'value'=>getRequest('forwardsysloghost','')]); }
				if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_PORT}'){API::UserMacro()->update(['hostmacroid'=>$macro['hostmacroid'],'value'=>getRequest('forwardsyslogport','')]); }
				if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_MODULE}'){API::UserMacro()->update(['hostmacroid'=>$macro['hostmacroid'],'value'=> getRequest('forwardsyslogprotocol',0)==1 ? 'om_tcp' : 'om_udp' ]); }
				if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_FOO}'){API::UserMacro()->update(['hostmacroid'=>$macro['hostmacroid'],'value'=> $macro['value']=='' ? 'nejaka nahodna hodnota, aby doslo k reload konfigurace': '' ]); }
			}		
			$itemSyslogForward = API::Item()->get([
				'output' => ['itemid','key_','status'], //API_OUTPUT_EXTEND,
				'search' => [
					'key_' => 'SyslogForwarder'
				],
				'hostids' => $templates[0]['templateid'],
				'limit' => 10
			]);	
			foreach($itemSyslogForward as $item){			
				API::Item()->update(['itemid'=>$item['itemid'],'status'=>1]);
			};
			show_messages(true, _('Syslog forwarding was disabled'), '');
		}
	}
}

/*
 * Display
 */

function getSyslogConf($conf){
	$syslog = [];
	
	$templates = API::Template()->get([
			'output' => API_OUTPUT_EXTEND,
			'search' => [
				'name' => $conf['templateServer']
			],
			'selectMacros' => API_OUTPUT_EXTEND,
			'limit' => 1
	]);
	if(count($templates)==1){
		$syslog['enabled'] = false;
		$templateid = $templates[0]['templateid'];
		$macros = $templates[0]['macros'];
		$itemAlarm = API::Item()->get([
			'output' => ['itemid','key_','status'], //API_OUTPUT_EXTEND,
			'search' => [
				'key_' => $conf['itemAlarm']
			],
			'hostids' => $templateid,
			'limit' => 10
		]);
		if(count($itemAlarm)==1){
			$syslog['enabled'] = $itemAlarm[0]['status'] == 0 ? true : false;
			foreach($macros as $macro){
				if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_HOST}'){$syslog['host'] = $macro['value']; }
				if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_PORT}'){$syslog['port'] = $macro['value']; }
				if($macro['macro']=='{$XLOG.AGENT.SYSLOG_FORWARD_MODULE}'){$syslog['protocol'] = $macro['value'] == 'om_tcp' ? 1 : 0; }
			}
		}
	}	
	return $syslog;
};	


$data = [];
$data['syslog'] = getSyslogConf($conf);
$data['backurl'] = $page['file'];
$forwardView = new CView('elisa.forward', $data);
$forwardView->render();
$forwardView->show();




require_once dirname(__FILE__).'/include/page_footer.php';
