<?php
/*
** Zabbix
** Copyright (C) 2001-2015 Zabbix SIA
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

ini_set('memory_limit', '4G');

define('ELISA_CONF','/usr/share/elisa/conf/elisa.conf');
define('ELISA_CONF_GUI','/usr/share/elisa/conf/elisa.gui.conf');


$elasticsearchAPI = shell_exec('grep -i ELASTICSEARCH_API= '.ELISA_CONF.' | cut -d= -f2 | cut -d# -f1');
//$elasticsearchAPI = "http://localhost:9500";
$DATA_PATH = shell_exec('grep -i -e "^DATA_PATH=" '.ELISA_CONF.' | cut -d= -f2 | cut -d# -f1');
$DATA_PATH_COMPRESS = shell_exec('grep -i DATA_PATH_COMPRESS= '.ELISA_CONF.' | cut -d= -f2 | cut -d# -f1');
$RAW_DATA_PATH = shell_exec('grep -i RAW_DATA_PATH= '.ELISA_CONF.' | cut -d= -f2 | cut -d# -f1');
$INDEX_INTEGRITY_KEY = shell_exec('grep -i INDEX_INTEGRITY_KEY= '.ELISA_CONF.' | cut -d= -f2 | cut -d# -f1');
$DISABLE_INDICE_DELETE = shell_exec('grep -i DISABLE_INDICE_DELETE= '.ELISA_CONF.' | cut -d= -f2 | cut -d# -f1');
define('elasticsearchAPI',trim($elasticsearchAPI));
define('DATA_PATH',trim($DATA_PATH));
define('DATA_PATH_COMPRESS',trim($DATA_PATH_COMPRESS));
define('DATAESS_PATH',str_replace("/raw","/nodes",trim($RAW_DATA_PATH)));
define('INDEX_INTEGRITY_KEY',trim($INDEX_INTEGRITY_KEY));
define('DISABLE_INDICE_DELETE',trim($DISABLE_INDICE_DELETE));

require_once dirname(__FILE__).'/include/config.inc.php';
require_once dirname(__FILE__).'/include/js.inc.php';
//require_once dirname(__FILE__).'/include/defines.inc.php';
//require_once dirname(__FILE__).'/include/audit.inc.php';

$page['title'] = _('ELISA DB Indices');
$page['file'] = 'elisa_indices.php';
$page['hist_arg'] = array('config');

require_once dirname(__FILE__).'/include/page_header.php';

//	VAR		TYPE	OPTIONAL FLAGS	VALIDATION	EXCEPTION
$fields = array(
	'indicesid' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
	'mindicesid' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
	'action' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
									IN('"indices.massopen","indices.massclose","indices.massarchive","indices.massrestore","indices.massdelete","indices.retentionmanual","indices.retentionauto","indices.masscreate"'),
									null
								],
	'sort' =>					[T_ZBX_STR, O_OPT, P_SYS,
									IN('"name","creationDate","status","retention"'),
									null
								],
	'sortorder' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null],
	'massupdate' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
	'bucket' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
	'cancel' =>					[T_ZBX_STR, O_OPT, P_SYS,			null,		null],
	'form' =>					[T_ZBX_STR, O_OPT, P_SYS,			null,		null]
);
check_fields($fields);



/*
 * Permissions
 */
 
$dbUserGroup = API::UserGroup()->get([
	'filter' => [
		'name' => 'ELISA-Indices'
	],
	'output' => API_OUTPUT_EXTEND
]);

if (!$dbUserGroup) {
	access_deny();
}

/*
 * Actions
 */


if (hasRequest('action') && getRequest('action') == 'indices.massopen' && hasRequest('indicesid')) {
	$indicesIds = getRequest('indicesid');
	zbx_value2array($indicesIds);
	$rc = true;
	foreach ($indicesIds as $name => $value) {
		exec('sudo /usr/share/elisa/bin/elisa-indiceOpen.sh '.$name, $output, $rci);
		if($rci > 0){$rc = false;}
		add_audit_details(AUDIT_ACTION_ENABLE, 99, 0, $name, 'indices');
	}
	show_messages($rc, _('Indices opened'), _('Cannot open indices'));
	uncheckTableRows('indicesid');	
	sleep(2);
}
elseif (hasRequest('action') && getRequest('action') == 'indices.massclose' && hasRequest('indicesid')) {
	$indicesIds = getRequest('indicesid');
	zbx_value2array($indicesIds);
	$rc = true;
	foreach ($indicesIds as $name => $value) {
		exec('sudo /usr/share/elisa/bin/elisa-indiceClose.sh '.$name, $output, $rci);
		if($rci > 0){$rc = false;}
		add_audit_details(AUDIT_ACTION_DISABLE, 99, 0, $name, 'indices');
	}
	show_messages($rc, _('Indices closed'), _('Cannot close indices'));	
	uncheckTableRows('indicesid');
	sleep(2);
}
elseif (hasRequest('action') && getRequest('action') == 'indices.massarchive' && hasRequest('indicesid')) {
	$NFS = explode(' ',shell_exec('df -h '.DATA_PATH_COMPRESS.' | grep '.DATA_PATH_COMPRESS.' | tr -s " " " " '));
	if(count($NFS)==6){
		$indicesIds = getRequest('indicesid');
		zbx_value2array($indicesIds);
		$rc = true;
		$cmds1 = [];
		$cmds2 = [];
		foreach ($indicesIds as $name => $value) {
			$cmds1[] = 'sudo /usr/share/elisa/bin/elisa-indiceArchiveTouchWaiting.sh '.$name;
			$cmds2[] = 'sudo /usr/share/elisa/bin/elisa-indiceArchive.sh '.$name;
			add_audit_details(AUDIT_ACTION_ADD, 99, 0, $name, 'indices-archive');
		}
		$cmd1 = implode(";",$cmds1);
		$cmd1 = '('.$cmd1.') >/dev/null 2>/dev/null';
		$cmd2 = implode(";",$cmds2);
		$cmd2 = '('.$cmd2.') >/dev/null 2>/dev/null &';
		exec($cmd1, $output, $rci);
		exec($cmd2, $output, $rci);
		if($rci > 0){$rc = false;}
		show_messages($rc, _('Indices started to be archived. Wait few minutes...'), _('Cannot archive indices'));	
		uncheckTableRows('indicesid');
		sleep(2);
	}else{
		show_messages(false, '', _('NFS share for storing data archive is not configured'));	
		uncheckTableRows('indicesid');
	}
}
elseif (hasRequest('action') && getRequest('action') == 'indices.massrestore' && hasRequest('indicesid')) {
	$indicesIds = getRequest('indicesid');
	zbx_value2array($indicesIds);
	$rc = true;
	$cmds1 = [];
	$cmds2 = [];
	foreach ($indicesIds as $name => $value) {
		$cmds1[] = 'sudo /usr/share/elisa/bin/elisa-indiceRestoreTouchWaiting.sh '.$name;
		$cmds2[] = 'sudo /usr/share/elisa/bin/elisa-indiceRestore.sh '.$name;
	}
	$cmd1 = implode(";",$cmds1);
	$cmd1 = '('.$cmd1.') >/dev/null 2>/dev/null';
	$cmd2 = implode(";",$cmds2);
	$cmd2 = '('.$cmd2.') >/dev/null 2>/dev/null &';
	exec($cmd1, $output, $rci);
	exec($cmd2, $output, $rci);
	if($rci > 0){$rc = false;}
	show_messages($rc, _('Indices started to be restores. Wait few minutes...'), _('Cannot restore indices'));	
	uncheckTableRows('indicesid');
	sleep(2);
}
elseif (hasRequest('action') && getRequest('action') == 'indices.massdelete' && hasRequest('indicesid')) {
	if( DISABLE_INDICE_DELETE != "TRUE" ){
		$indicesIds = getRequest('indicesid');
		zbx_value2array($indicesIds);
		$rc = true;
		foreach ($indicesIds as $name => $value) {
			exec('(sudo /usr/share/elisa/bin/elisa-indiceDelete.sh -f '.$name.') >/dev/null 2>/dev/null &', $output, $rci);
			if($rci > 0){$rc = false;}
			add_audit_details(AUDIT_ACTION_DELETE, 99, 0, $name, 'indices');
		}
		show_messages($rc, _('Indices started to be deleted. Wait few minutes...'), _('Cannot delete indices'));	
		uncheckTableRows('indicesid');
		sleep(2);
	}
}
elseif (hasRequest('action') && getRequest('action') == 'indices.retentionmanual' && hasRequest('indicesid')) {
	$indicesIds = getRequest('indicesid');
	zbx_value2array($indicesIds);
	$rc = true;
	foreach ($indicesIds as $name => $value) {
		exec('sudo /usr/share/elisa/bin/elisa-indiceRetentionMode.sh '.$name.' manual', $output, $rci);	
		if($rci > 0){$rc = false;}
		add_audit_details(AUDIT_ACTION_DISABLE, 99, 0, $name, 'indices-retention');
	}
	show_messages($rc, _('Indices retention mode has been set'), _('Cannot set retention mode'));	
	uncheckTableRows('indicesid');
}
elseif (hasRequest('action') && getRequest('action') == 'indices.retentionauto' && hasRequest('indicesid')) {
	$indicesIds = getRequest('indicesid');
	zbx_value2array($indicesIds);
	$rc = true;
	foreach ($indicesIds as $name => $value) {
		exec('sudo /usr/share/elisa/bin/elisa-indiceRetentionMode.sh '.$name.' auto', $output, $rci);	
		if($rci > 0){$rc = false;}
		add_audit_details(AUDIT_ACTION_ENABLE, 99, 0, $name, 'indices-retention');
	}
	show_messages($rc, _('Indices retention mode has been set'), _('Cannot set retention mode'));	
	uncheckTableRows('indicesid');
}
elseif (hasRequest('action') && getRequest('action') == 'indices.masscreate' && hasRequest('mindicesid')) {
	$indicesIds = getRequest('mindicesid');
	zbx_value2array($indicesIds);
	$rc = true;
	foreach ($indicesIds as $name => $value) {
		exec('(sudo /usr/share/elisa/bin/elisa-indiceCreate.sh '.$name.') >/dev/null 2>/dev/null &', $output, $rci);
		if($rci > 0){$rc = false;}
		add_audit_details(AUDIT_ACTION_ADD, 99, 0, $name, 'indices');
	}
	show_messages($rc, _('Indices started to be created. Wait few minutes...'), _('Cannot create indices'));	
	uncheckTableRows('mindicesid');
	sleep(2);
}


/*
 * Display
 */

	//$bucket = getRequest("bucket");

	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
	$sortOrder = getRequest('sortorder', CProfile::get('web.'.$page['file'].'.sortorder', ZBX_SORT_DOWN));
	$bucket = getRequest('bucket', CProfile::get('web.'.$page['file'].'.bucket', ''));
	
	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
	CProfile::update('web.'.$page['file'].'.bucket', $bucket, PROFILE_TYPE_STR);
	
	$indices = array(
		'sort' => $sortField,
		'sortorder' => $sortOrder,
		'storage' => array()
	);
	
	$indices['bucket']['show'] = $bucket;
	$indices['bucket']['config'] = [];
	$file = fopen(ELISA_CONF_GUI,"r");
	while(! feof($file)){
		$line = fgets($file);
		if(!is_int(strpos($line, '#')) && trim($line) != ""){
			$lineArr = explode("=",$line);
			$key = trim($lineArr[0]);
			$value = trim($lineArr[1]);
			if(strpos($key, 'DATA_BUCKET_') !== false){
				$keyArr = explode("_",$key);
				$key_index = trim($keyArr[2]);
				$key_name = trim($keyArr[3]);
				$indices['bucket']['config'][$key_index][$key_name] = $value;
			}
		}
	}
	fclose($file);	
	
	$indices['storage']['Data ESM'] = getStorageInfo(DATA_PATH);
	$storageESS=getStorageInfo(DATAESS_PATH);
	if( $indices['storage']['Data ESM']['mount'] != $storageESS['mount'] ){
		$indices['storage']['Data ESS'] = getStorageInfo(DATAESS_PATH);
	}
	#$indices['storage']['Archive'] = getStorageInfo(DATA_PATH_COMPRESS);
	

	$indiceURI = elasticsearchAPI.'/_cat/indices/elisa'.$bucket.'-*.*.*?v&h=index,status&format=json&s=index:desc';
	//$indiceURI = elasticsearchAPI.'/_cat/indices/*-*.*.*?v&bytes=b&h=index,uuid,status,docs.count,store.size,creation.date,creation.date.string&format=json&s=index:desc';
	$indiceURIresponse = json_request($indiceURI, 'GET', null);
	
	$aliasURI = elasticsearchAPI.'/_cat/aliases/metahash*?v&h=index,alias&format=json';
	$aliasURIresponse = json_request($aliasURI, 'GET', null);	
	$aliasMeta = [];
	if(isset($aliasURIresponse)){
		foreach($aliasURIresponse as $row){
			$alias = preg_replace("/^.*-/", "", $row['alias']);
			$aliasMeta[$row['index']] = $alias;
		}
	}
	
	$dbdata = DBfetchArray(DBselect(
		'SELECT indice,uuid,creationdate,size,events FROM elisa_esindices'
	));
	$dbdatakeys = [];
	foreach($dbdata as $row){ $dbdatakeys[] = $row['indice']; }
	$dbdata = array_combine($dbdatakeys,$dbdata);

	$indices['indice'] = array();
	
	if(isset($indiceURIresponse)){
		foreach($indiceURIresponse as $row){
			$name = $row['index'];
			$indices['indice'][$name]['name'] = $row['index'];
			$indices['indice'][$name]['status'] = $row['status'];
			$indices['indice'][$name]['retention'] = 'auto';
			$indices['indice'][$name]['tmpHash'] = null;
			$indices['indice'][$name]['metaHash'] = null;
			if( isset($aliasMeta[$name]) ){ $indices['indice'][$name]['metaHash'] = $aliasMeta[$name]; }
			$indices['indice'][$name]['dataHash'] = null;
			$indices['indice'][$name]['realMetaHash'] = '';
			$indices['indice'][$name]['realDataHash'] = '';
			if(array_key_exists($name,$dbdata)){
				$indices['indice'][$name]['creationDate'] = $dbdata[$name]['creationdate'];
				$indices['indice'][$name]['size_in_bytes'] = $dbdata[$name]['size'];	
				$indices['indice'][$name]['docs_count'] = $dbdata[$name]['events'];
			}else{
				if($row['status']=="open"){
					$indexProperties = getEsProperties($name);
					if( ! array_key_exists("error",$indexProperties) && count($indexProperties)==1 ){
						$indices['indice'][$name]['creationDate'] = $indexProperties[0]['creation.date'];
						$indices['indice'][$name]['size_in_bytes'] = $indexProperties[0]['store.size'];		
						$indices['indice'][$name]['docs_count'] = $indexProperties[0]['docs.count'];
					}
					
				}else{
					$indices['indice'][$name]['creationDate'] = null;
					$indices['indice'][$name]['size_in_bytes'] = null;		
					$indices['indice'][$name]['docs_count'] = null;
				}
			}
	
			
			if($row['status'] == "open"){
				$indices['indice'][$name]['realMetaHash'] = countMetaHash($name);
			/*}else if($row['status'] == "close"){
				$output=[];
				exec('/usr/bin/du -sb '.DATA_PATH.'/'.$row['uuid']. ' | cut -f1', $output, $rci);
				if(count($output)==1){
					$indices['indice'][$name]['size_in_bytes'] = $output[0];
				};	*/		
			};			
		}
	}else{
		$indices['indice'] = array();
		show_messages(false, '', _('Elasticsearch API is not responding'));	
	}
	

/*
	$dataDirs = scandir(DATA_PATH_COMPRESS);
	foreach ($dataDirs as $file) {
		//if(strpos($file, 'logstash') !== FALSE && strpos($file, 'logstash') == 0 && is_file(DATA_PATH_COMPRESS.'/'.$file)){
		if( preg_match( '/.*-\d\d\d\d\.\d\d.\d\d\.tgz$/', $file) && is_file(DATA_PATH_COMPRESS.'/'.$file)){
			$fileinfo = pathinfo(DATA_PATH_COMPRESS.'/'.$file);
			$fileName = $fileinfo['filename'];
			$fileExt = $fileinfo['extension'];
			$fileSize = filesize(DATA_PATH_COMPRESS.'/'.$file);
			if($fileSize){ $fileSize = $fileSize; }
			if(! array_key_exists($fileName,$indices['indice']) ){ 
				$indices['indice'][$fileName]['name'] = $fileName;
				$indices['indice'][$fileName]['creationDate'] = filemtime(DATA_PATH_COMPRESS.'/'.$file)*1000;
				$indices['indice'][$fileName]['status'] = 'archive';
				$indices['indice'][$fileName]['retention'] = 'auto';
				$indices['indice'][$fileName]['tmpHash'] = null;
				$indices['indice'][$fileName]['metaHash'] = null;
				$indices['indice'][$fileName]['dataHash'] = null;
				$indices['indice'][$fileName]['realMetaHash'] = '';
				$indices['indice'][$fileName]['realDataHash'] = '';
				$indices['indice'][$fileName]['size_in_bytes'] = $fileSize;
			}
		}
	}	
*/	
	//nacist status prave se kompresujicich a dekompresujicich indexu
	$dataDirs = scandir(DATA_PATH);
	foreach ($dataDirs as $flagFile) {
		if( preg_match( '/^\..*-\d\d\d\d\.\d\d\.\d\d\..*$/', $flagFile) && strpos($flagFile, 'state') !== FALSE && is_file(DATA_PATH.'/'.$flagFile)){
			$fileinfo = pathinfo(DATA_PATH.'/'.$flagFile);
			if( preg_match( '/^\.(.*-\d\d\d\d\.\d\d.\d\d)\..*$/', $flagFile, $matches) ){
				$indiceName = $matches[1];
				$indiceState = $fileinfo['extension'];
				if(array_key_exists($indiceName,$indices['indice']) && $indiceState != ""){ 
					$indices['indice'][$indiceName]['status'] = $indiceState;
				}
			}
		}
		else if( preg_match( '/^\..*-\d\d\d\d\.\d\d.\d\d\..*$/', $flagFile) && strpos($flagFile, 'retention') !== FALSE && is_file(DATA_PATH.'/'.$flagFile) ){
			$fileinfo = pathinfo(DATA_PATH.'/'.$flagFile);
			if( preg_match( '/^\.(.*-\d\d\d\d\.\d\d.\d\d)\..*$/', $flagFile, $matches) ){
				$indiceName = $matches[1];
				$indiceRetention = $fileinfo['extension'];
				if(array_key_exists($indiceName,$indices['indice']) && $indiceRetention != ""){ 
					$indices['indice'][$indiceName]['retention'] = $indiceRetention;
				}
			}
		}
	}	

	order_result($indices['indice'], $sortField, $sortOrder);
	//var_dump($indices['indice']);

	if( DISABLE_INDICE_DELETE == "TRUE" ){ $indices['DISABLE_INDICE_DELETE']=true; }else{ $indices['DISABLE_INDICE_DELETE']=false; }
	
	if (getRequest('form')=="create") {
		$indicesView = new CView('elisa.indices.edit', $indices);
	}else{
		$indices['paging'] = getPagingLine($indices['indice'], $sortOrder, new CUrl('elisa_indices.php'));
		// render view
		$indicesView = new CView('elisa.indices.list', $indices);
	}
	$indicesView->render();
	$indicesView->show();
	


	

require_once dirname(__FILE__).'/include/page_footer.php';

function getEsProperties($indiceName){
	$indiceURI = elasticsearchAPI."/_cat/indices/$indiceName?v&bytes=b&h=index,uuid,status,docs.count,store.size,creation.date,creation.date.string&format=json&s=index:desc";
	$indiceURIresponse = json_request($indiceURI, 'GET', null);
	return $indiceURIresponse;
}

function countMetaHash($indiceName){
	//$hashCmd = "cd ".DATA_PATH."/".$indiceName."/ && find ./*/index ./*/translog -type f -exec sha256sum {} + | awk 'BEGIN {print ".INDEX_INTEGRITY_KEY."} {print $1}' | sort | sha256sum | awk '{print $1}'";
	//$hashCmd = "ls -laR ".DATA_PATH."/".$indiceName."/*/index/_* | awk 'BEGIN {print ".INDEX_INTEGRITY_KEY."} {print $0}' | sort | sha256sum | awk '{print $1}'";
	$indiceStatusURI = elasticsearchAPI.'/'.$indiceName.'/_stats/docs?filter_path=_all.total.docs';
	#$hashCmd = "curl --noproxy \"localhost 127.0.0.1\" -s -XGET '$indiceStatusURI' | awk 'BEGIN {print ".INDEX_INTEGRITY_KEY."} {print $0}' | sort | sha256sum | awk '{print $1}'";
	$hashCmd = "{ curl --noproxy \"localhost 127.0.0.1\" -s -XGET '$indiceStatusURI'; echo ".INDEX_INTEGRITY_KEY."; } | tr \"\\n\" \" \" | sort | sha256sum | awk '{print $1}'";
	//echo "$hashCmd<br/>";
	exec($hashCmd, $output, $rci);	
	return trim($output[0]);
}


function getStorageInfo($path){
	$storage = [];
	$output=[];
	exec('/usr/bin/df -h '.$path.'', $output, $rci);
	if(count($output)==2){
		$arrOutput = preg_split('/\s+/', $output[1]);		
		$storage['total'] = $arrOutput[1];
		$storage['used'] = $arrOutput[2];
		$storage['free'] = $arrOutput[3];
		$storage['usedperc'] = $arrOutput[4];
		$storage['mount'] = $arrOutput[5];
		$storage['dirsize'] = shell_exec('du -sh '.$path.' | cut -s -f1');		
	};
	return $storage;
}


function json_request($uri, $requesttype, $data) {
	$json_data = json_encode($data);	
	$c = curl_init();
	curl_setopt($c, CURLOPT_URL, $uri);
	curl_setopt($c, CURLOPT_CUSTOMREQUEST, $requesttype); 
	if($requesttype == "POST"){
		curl_setopt($c, CURLOPT_POST, $json_data);
		curl_setopt($c, CURLOPT_POSTFIELDS, $json_data);
		curl_setopt($c, CURLOPT_HTTPHEADER, array(                                                                          
			'Content-Type: application/json',                                                                                
			'Content-Length: ' . strlen($json_data))                                                                       
		);
	};
	if($requesttype == "PUT"){
		//curl_setopt($c, CURLOPT_PUT, true);
		curl_setopt($c, CURLOPT_PUT, true);
		curl_setopt($c, CURLOPT_HTTPHEADER, array('X-HTTP-Method-Override: PUT'));
	}
	curl_setopt($c, CURLOPT_RETURNTRANSFER, true); 
	curl_setopt($c, CURLOPT_SSL_VERIFYHOST, false);
	curl_setopt($c, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($c, CURLOPT_VERBOSE, false);

	$result = curl_exec($c);
	
	// Uncomment to see some debug info
	/*echo "<b>JSON Request:</b><br>\n";
	echo $json_data."<br><br>\n";

	echo "<b>JSON Answer:</b><br>\n";
	echo $result."<br><br>\n";

	echo "<b>CURL Debug Info:</b><br>\n";
	$debug = curl_getinfo($c);
	echo expand_arr($debug)."<br><hr>\n";
	*/

	return json_decode($result, true);
}


