<?php
/*
** Zabbix
** Copyright (C) 2001-2015 Zabbix SIA
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

ini_set('memory_limit', '4G');


require_once dirname(__FILE__).'/include/config.inc.php';
require_once dirname(__FILE__).'/include/js.inc.php';
require_once dirname(__FILE__).'/include/func.inc.php';
require_once dirname(__FILE__).'/include/defines.inc.php';
//require_once dirname(__FILE__).'/include/audit.inc.php';

$page['title'] = _('ELISA Logsources');
$page['file'] = 'elisa_logsources.php';
$page['hist_arg'] = array('config');
$page['scripts'] = ['class.calendar.js'];

//CView::$has_web_layout_mode = true;
//$page['web_layout_mode'] = CView::getLayoutMode();



//	VAR		TYPE	OPTIONAL FLAGS	VALIDATION	EXCEPTION
$fields = array(
	'sort' =>					[T_ZBX_STR, O_OPT, P_SYS,
									IN('"host","input","creationdate","lastdate","deadPerc"'),
									null
								],
	'sortorder' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null],
	'filter_set' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
	'filter_rst' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,		null],	
	'filter_host_value' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
	'filter_input_value' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
	'filter_nodata_value' =>	[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
	/*'filter_lasteventdate_value' =>			[T_ZBX_STR, O_OPT, null,	null,		null],*/
	'cancel' =>					[T_ZBX_STR, O_OPT, P_SYS,			null,		null],
	'form' =>					[T_ZBX_STR, O_OPT, P_SYS,			null,		null],
	'kioskmode'	 =>				[T_ZBX_INT, O_OPT, null,	IN('0,2'),	null],
	'action' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
									IN('"remove"'),
									null
								],
	'hostid' =>					[T_ZBX_INT, O_OPT, null,	null,		null],
	'source' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
	'hostname' =>				[T_ZBX_STR, O_OPT, null,	null,		null]
);
check_fields($fields);

if(getRequest('kioskmode', 0) == 2 ){
  $page['web_layout_mode'] = ZBX_LAYOUT_KIOSKMODE;	
}
require_once dirname(__FILE__).'/include/page_header.php';

/*
 * Permissions
 */
 
$zabbix_hosts = API::Host()->get([
	'output' => ['hostid']
]);
$zabbix_hosts_id = "0";
foreach($zabbix_hosts as $i => $host){ 
	$zabbix_hosts_id = $zabbix_hosts_id.','.$host['hostid'];
}

$zabbix_hosts_admin = API::Host()->get([
	'output' => ['hostid'],
	'editable' => true
]);
$zabbix_hosts_admin_arr = [];
foreach($zabbix_hosts_admin as $i => $host){ $zabbix_hosts_admin_arr[] = $host['hostid']; }



/*
 * Actions
 */

if (hasRequest('action') && getRequest('action') == 'remove' && hasRequest('hostid') && hasRequest('source')) {
	$hostid = getRequest('hostid',0);
	$input = getRequest('source');
	$hostname = getRequest('hostname');
	
	if(in_array($hostid, $zabbix_hosts_admin_arr)) {
		$result = DBexecute('delete from elisa_logsources where hostid='.$hostid.' and input=\''.$input.'\'');
		if( $result ){
			add_audit_details(AUDIT_ACTION_DELETE, 99, 0, $hostname.'-'.$input, 'logsource');
			show_messages(true, _('Item deleted'), '');
		}else{
			show_messages(false, '', _('No permissions to referred object or it does not exist!'));			
		}		
	}else{
		show_messages(false, '', _('You do not have permission to perform this operation.'));
	}
	
}


/*
 * Display
 */

$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'deadPerc'));
$sortOrder = getRequest('sortorder', CProfile::get('web.'.$page['file'].'.sortorder', ZBX_SORT_DOWN));
	
	$data = array(
		'sort' => $sortField,
		'sortorder' => $sortOrder,
		'sources' => [],
		'profileIdx' => 'elisa.logsources.filter',
		'active_tab' => CProfile::get('elisa.logsources.filter.active', 1),
		'getLayoutMode' => getRequest('kioskmode', 0)
	);


	/*
	 * Filter
	 */
	if (hasRequest('filter_set')) {
		CProfile::update('elisa.logsources.filter_host_value', getRequest('filter_host_value', ''), PROFILE_TYPE_STR);
		CProfile::update('elisa.logsources.filter_input_value', getRequest('filter_input_value', ''), PROFILE_TYPE_STR);
		/*CProfile::update('elisa.logsources.filter_nodata_value', getRequest('filter_nodata_value', 0), PROFILE_TYPE_INT);*/
		CProfile::update('elisa.logsources.filter_lasteventdate_value', getRequest('filter_lasteventdate_value', ''), PROFILE_TYPE_STR);
	}
	elseif (hasRequest('filter_rst')) {
		DBStart();
		CProfile::delete('elisa.logsources.filter_host_value');
		CProfile::delete('elisa.logsources.filter_input_value');
		/*CProfile::delete('elisa.logsources.filter_nodata_value');*/
		CProfile::delete('elisa.logsources.filter_lasteventdate_value');
		DBend();
	}

	$data['filterHostValue'] = CProfile::get('elisa.logsources.filter_host_value', '');
	$data['filterInputValue'] = CProfile::get('elisa.logsources.filter_input_value', '');
	/*$data['filterNodataValue'] = CProfile::get('elisa.logsources.filter_nodata_value', 0);*/
	$data['filterLasteventdateValue'] = CProfile::get('elisa.logsources.filter_lasteventdate_value', '');
	
	

	
	$where = ' where hostid in ('.$zabbix_hosts_id.')';
	if($data['filterHostValue']!=''){ $where = $where.' and lower(host) like \'%'.strtolower($data['filterHostValue']).'%\' '; }
	if($data['filterInputValue']!=''){ $where = $where.' and lower(input) like \'%'.strtolower($data['filterInputValue']).'%\' '; }
	/*if($data['filterNodataValue']==0){ $where = $where.' and lastdate is not null'; }*/
	if($data['filterLasteventdateValue']!=''){
		$range_time_parser = new CRangeTimeParser();
		$range_time_parser->parse($data['filterLasteventdateValue']);
		//var_dump($range_time_parser);
		$filterLasteventdateValue = $range_time_parser->getDateTime(true)->getTimestamp();
		$where = $where.' and lastdate < '.$filterLasteventdateValue; 
	}
	$dbdata = DBfetchArray(DBselect(
		'SELECT * FROM elisa_logsources'.$where
	));

$now = time();
foreach($dbdata as $i => $row){ 
	$deadintervalsec = timeUnitToSeconds($row['deadinterval']);
	$deadPerc = 0;
	if( $deadintervalsec != "" ){
		$dbdata[$i]['deadintervalsec'] = $deadintervalsec;
		if($row['lastdate']>0){
			$deadInSec = $now-$row['lastdate'];
			$deadPerc = round($deadInSec/$deadintervalsec*100,0);
		}
	}else{
		$dbdata[$i]['deadintervalsec'] = -1;
	}
	$dbdata[$i]['deadPerc'] = $deadPerc;
	if (in_array($row['hostid'], $zabbix_hosts_admin_arr)) {
		$dbdata[$i]['admin'] = true;
	}else{
		$dbdata[$i]['admin'] = false;
	}
	
}
//var_dump($dbdata);

$data['sources'] = $dbdata;
order_result($data['sources'], $sortField, $sortOrder);



	$data['paging'] = getPagingLine($data['sources'], $sortOrder, new CUrl('elisa_logsources.php'));
		// render view
	$sourcesView = new CView('elisa.logsources', $data);

	$sourcesView->render();
	$sourcesView->show();	

require_once dirname(__FILE__).'/include/page_footer.php';


