<?php
/*
** Zabbix
** Copyright (C) 2001-2015 Zabbix SIA
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/



require_once dirname(__FILE__).'/include/config.inc.php';
require_once dirname(__FILE__).'/include/js.inc.php';

$page['title'] = _('ELISA System');
$page['file'] = 'elisa_system.php';
$page['type'] = detect_page_type(PAGE_TYPE_HTML);

#define('ZBX_PAGE_DO_REFRESH', 1);

require_once dirname(__FILE__).'/include/page_header.php';


//	VAR		TYPE	OPTIONAL FLAGS	VALIDATION	EXCEPTION
$fields = array(
	'rebootserver'=>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null),
	'restartservice'=>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null)
);
//check_fields($fields);



/*
 * Permissions
 */
 

/*
 * Actions
 */

if (hasRequest('rebootserver')) {
	exec('(/usr/bin/sleep 5 && sudo /usr/sbin/reboot now) >/dev/null 2>/dev/null &', $output, $rc1);
	add_audit_details(AUDIT_ACTION_UPDATE, 99, 0, 'OS REBOOT', 'system');
	show_messages(true, _('Server reboot initiated. It can take few minutes.'), _('Error occured during initiating server reboot.'));	
}else if (hasRequest('restartservices')) {
	exec('sudo /usr/share/elisa/bin/elisa-restart.sh >/dev/null 2>/dev/null &', $output, $rc1);
	add_audit_details(AUDIT_ACTION_UPDATE, 99, 0, 'service restart', 'system');
	show_messages(true, _('System services restart initiated. It can take few minutes.'), _('Error occured during initiating services restart.'));	
}

/*
 * Display
 */



function getServiceStatus($servicename){
	$status = 3;
	$output = shell_exec("/usr/bin/systemctl status $servicename | grep running | wc -l");
	$output = trim($output);
	if($output=="1"){$status = 1;}
	else{
		$output = shell_exec("/usr/bin/systemctl status $servicename | grep enabled | wc -l");
		$output = trim($output);
		if($output!="0"){$status = 0;}
		else{$status = 2;}
	}
	return $status;
} 

//var_dump($data);

function writeServiceStatus($val){
	if($val==1){
		return (new CSpan(_('Running')))->AddClass('green');		
	}else if($val==0){
		return (new CSpan(_('Stopped')))->AddClass('red');		
	}else if($val==2){
		return (new CSpan(_('Disabled')))->AddClass('grey');		
	}else{
		return (new CSpan(_('Unknown')))->AddClass('yellow');		
	}
}

$data = [];
$data['service']['Elasticsearch ESM'] = getServiceStatus('elasticsearch.service');
$data['service']['Elasticsearch ESS'] = getServiceStatus('elasticsearch-ess.service');
$data['service']['NXlog'] = getServiceStatus('nxlog.service');
$data['service']['Memcached'] = getServiceStatus('memcached.service');
$data['service']['Zabbix Server'] = getServiceStatus('zabbix-server.service');
$data['service']['Zabbix Agent'] = getServiceStatus('zabbix-agent.service');



// render view
$systemView = (new CWidget())->setTitle(_('ELISA System'));
	$systemTab = new CTabView();
	$systemView->addItem(new CTag('neco'));

		$servicesDiv = new CDiv();
		$servicesDiv->addItem((new CTag('h3',true))->addItem(_('Services status')));
		$servicesDiv->addItem((new CTag('br')));
		
		$servicesTable = (new CTableInfo())->AddStyle('width:auto;');
		foreach ($data['service'] as $title => $status) {
			$servicesTable->addRow([(new CSpan($title)),writeServiceStatus($status)]);
		}

		$servicesDiv->addItem($servicesTable);
		
		$servicesForm = new CForm('post', null);
		
		function confirmScript($confirm_message){
			$confirm_script = ($confirm_message !== '')
				? 'Confirm('.CHtml::encode(CJs::encodeJson($confirm_message)).') && '
				: '';
			return $confirm_script;
		}
				
		$servicesForm->addItem(makeFormFooter(
			(new CSubmit('rebootserver', _('Reboot Server')))->onClick("javascript: return ".confirmScript(_('Confirm server reboot request please'))."submit;"),
			[(new CSubmit('restartservices', _('Restart Services')))->onClick("javascript: return ".confirmScript(_('Confirm services restart request please'))."submit;")]
		));
		$servicesDiv->addItem($servicesForm);
	
	$systemTab->addTab('services', _('Services'), $servicesDiv);

#$systemView->render();

$systemView->addItem($systemTab);
$systemView->show();	


# auto refresh page
$pageurl = $_SERVER['PHP_SELF'];
$sec = "5";
header("Refresh: $sec; url=$pageurl");


require_once dirname(__FILE__).'/include/page_footer.php';


